/* Copyright (C) 2003, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * Name.java
 *
 *
 */

package uiuc.Jumbo.Compiler;

import uiuc.Jumbo.Jaemus.*;

/**
 * A special kind of code for variable hygiene
 * @version      1.0,  19 Mar 2003
 * @author       Lars Clausen
 *
 */

public class Name extends Code {
    private String fullname;
    private String name;
    public Name(String n) {
        name = n;
    }
    public String getName() {
        if (fullname == null)
            throw Constructor.error("Getting name for undefined variable "+name, null);
        return fullname;
    }
    public ClassEnvironment define(Type t, ClassEnvironment env) {
        return define(t, 0, env);
    }
    public ClassEnvironment define(Type t, int f, ClassEnvironment env) {
        if (fullname == null) {
            final ClassEnvironment env1 = env.generateVar(name, t, f);
            final Local v = env1.getNewestVar();
            fullname = v.name;
            return env1;
        } else {
            throw env.error("Redefining variable "+name);
        }
    }
    public ClosedCode eval(Environment env) {
        if (fullname == null) {
            throw env.error("Using undefined variable "+name);
        } else {
            Var v = env.lookupVar(fullname);
            if (v != null) {
                if (v.isConstant()) {
                    return new ClosedCode(v.constantValue().genLoadCode(),
                                          env, v.type, v.constantValue());
                } else {
                    return new ClosedCode(Instr.genLoad(v.index, v.type),
                                          env, v.type);
                }
            } else {
                throw env.error("Internal:  Named variable is not defined: "+name+" named "+fullname);
            }
        }
    }
    public String toString() {
        return (fullname!=null?fullname:name);
    }
}
