/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * TestClasses.java
 *
 *
 */

package uiuc.Jumbo.Compiler;

import junit.framework.TestCase;
import uiuc.Jumbo.Jaemus.*;
import uiuc.Jumbo.Util.*;
import java.util.*;

/**
 * Unit tests for class creation and writing.
 * @version      1.0,  2 Nov 2001
 * @author       Lars Clausen
 *
*/

public class TestClasses extends TestCase {
    private static final Environment EMPTY_ENV = Environment.emptyEnv();

    public TestClasses(String name) {
        super(name);
    }
    
    public void testCreate() {
        Code d1 = Constructor.defineClass(0,
                                          "AClass",
                                          null,
                                          new ArrayMonoList(0),
                                          new ArrayMonoList(0));
        ClosedCode cc = d1.eval(d1.defineMembers(d1.defineSupers(d1.defineClasses(EMPTY_ENV))));
        ClassInfo c1 = cc.env.getClass("AClass");
        ClassInfo c2 = new ClassInfo("AClass");
        assertEquals("Checking trivial class equivalence", c1, c2);
    }
    public void testCreateField() {
        MonoList fields = new ArrayMonoList(1);
        fields.add(Constructor.defineField(Flags.PUBLIC|Flags.STATIC,
                                           Type.int_type,
                                           "foo", null));
        Code d1 = Constructor.defineClass(0,
                                          "AClass",
                                          null,
                                          new ArrayMonoList(0),
                                          fields);
        ClosedCode cc = d1.eval(d1.defineMembers(d1.defineSupers(d1.defineClasses(EMPTY_ENV))));
        ClassInfo c1 = cc.env.getClass("AClass");
        ClassInfo c2 = new ClassInfo("AClass");
        FieldInfo f1 = new FieldInfo(c2, Flags.PUBLIC|Flags.STATIC, "foo",
                                     Type.int_type);
        assertEquals("Checking class equivalence with field", c1, c2);
    }
}

/* End of class TestClasses*/
