/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * ConstantValueAttribute.java
 *
 *
 */

package uiuc.Jumbo.Jaemus;

import java.io.*;

/**
 * The ConstantValueAttribute holds a constant value of a field.
 * Particularly important for interfaces.
 * @version      1.0, 23 Jan 2002
 * @author       Lars Clausen
 *
*/

public class ConstantValueAttribute extends AttributeInfo {
    protected Object value;
    protected Constant valueconstant;

    ConstantValueAttribute(Object value) {
        super("ConstantValue");
        this.value = value;
        size = 2;
    }

    public void prepareForEmit(ConstantPool c) {
        super.prepareForEmit(c);
        if (value instanceof String)
            valueconstant = c.addConstant((String)value);
        else if (value instanceof Long)
            valueconstant = c.addConstant(((Long)value).longValue());
        else if (value instanceof Float)
            valueconstant = c.addConstant(((Float)value).floatValue());
        else if (value instanceof Double)
            valueconstant = c.addConstant(((Double)value).doubleValue());
        else if (value instanceof Number)
            valueconstant = c.addConstant(((Number)value).intValue());
        else if (value instanceof Boolean)
            valueconstant = c.addConstant((((Boolean)value).booleanValue()?1:0));
        else
          throw new Error("Illegal constant value: "+value+", type "+value.getClass());
    }

    public void emitTo(DataOutputStream o) throws IOException {
        emitHeader(o);
        valueconstant.emitIndex(o);
    }
}

/* End of class ConstantValueAttribute*/
