/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * Flags.java
 *
 *
 */

package uiuc.Jumbo.Jaemus;

/**
 * Repository for flags and flag-handling routines
 * @version      1.0,  8 Oct 2001
 * @author       Lars Clausen
 *
*/

public class Flags {
    public final static int PUBLIC = 0x0001;
    public final static int PRIVATE = 0x0002;
    public final static int PROTECTED = 0x0004;
    public final static int STATIC = 0x0008;
    public final static int FINAL = 0x0010;
    public final static int SYNCHRONIZED = 0x0020;
    public final static int SUPER = 0x0020;
    public final static int VOLATILE = 0x0040;
    public final static int TRANSIENT = 0x0080;
    public final static int NATIVE = 0x0100;
    public final static int INTERFACE = 0x0200;
    public final static int ABSTRACT = 0x0400;
    public final static int STRICTFP = 0x0800;

    static final String[] names = {
        "public",
        "private",
        "protected",
        "static",
        "final",
        "synchronized",
        //      "super",
        "volatile",
        "transient",
        "native",
        "interface",
        "abstract",
        "strictfp",
    };

    public static boolean bitsSet(int flagset, int bits) {
        return (flagset & bits) == bits;
    }

    public static String toString(int flagset) {
        String res = null;
        for (int i = 0; i < 12; i++) {
            if ((flagset & (1 << i)) != 0) {
                if (res == null) res = names[i];
                else res += " "+names[i];
            }
        }
        return (res==null?"":res);
    }

    public static int fromString(String s) {
        for (int i = 0; i < 12; i++) {
            if (names[i].equals(s)) return (1<<i);
        }
        return -1;
    }
}

/* End of class Flags*/
