/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * TestType.java
 *
 *
 */

package uiuc.Jumbo.Jaemus;

import junit.framework.TestCase;

import uiuc.Jumbo.Jaemus.*;
import java.util.*;

public class TestType extends TestCase {

    public TestType(String name) {
        super(name);
    }

    public void testIntType() {
        checkType( Type.int_type, "I" );
    }

    public void testByteType() {
        checkType( Type.byte_type, "B" );
    }

    public void testCharType() {
        checkType( Type.char_type, "C" );
    }

    public void testDoubleType() {
        checkType( Type.double_type, "D" );
    }

    public void testFloatType() {
        checkType( Type.float_type, "F" );
    }

    public void testLongType() {
        checkType( Type.long_type, "J" );
    }

    public void testShortType() {
        checkType( Type.short_type, "S" );
    }

    public void testBooleanType() {
        checkType( Type.boolean_type, "Z" );
    }

    public void testObjectType() {
        checkType( Type.object_type, "Ljava/lang/Object;" );
    }

    public void testStringType() {
        checkType( Type.string_type, "Ljava/lang/String;" );
    }

    public void testNullType() {
        checkType( Type.null_type, "Ljava/lang/Object;" );
    }

    public void testVoidType() {
        checkType( Type.void_type, "V" ); 
    }

    public void testAnyObjectType () {
        checkType( new Type( "foo/bar/oop/ack/fiddle/sticks" ), 
                   "Lfoo/bar/oop/ack/fiddle/sticks;" );
    }

    public void testOneDimArray() {
        checkArrayType( Type.int_type, 1 );
    }

    public void testTwoDimArray() {
        checkArrayType( Type.boolean_type, 2 );
    }

    public void testThreeDimArray() {
        checkArrayType( Type.float_type, 3 );
    }

    public void testFourDimArray() {
        checkArrayType( Type.long_type, 4 );
    }

    public void testSixteenDimArray() {
        checkArrayType( Type.object_type, 16 );
    }

    /*************** Convenience methods. ******************/

    private void checkType( Type t, String code ) {
        checkType( t, code, "" );
    }

    private void checkType( Type t, String code, String message ) {
        assertEquals("type: "+t+"; "+message+"\n", code, t.bytecodeString());
    }

    private void checkArrayType( Type t, int dimensions ) {
        Type a = t;
        String code = t.bytecodeString();

        for( int i = 0; i < dimensions; i++ ) {
            a = new Type(a);
            code = "[" + code;
        }

        checkType( a, code, ""+dimensions+"-dimensional array" );
    }
}
