/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.*;

/**
 * Represents a list of arguments passed to a method in an invocation
 *
 * @author      Baris Aktemur
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ArgumentList extends Node implements List{
    public ArgumentList() { 
	args = new LinkedList();
	setParents(args);
    }

    public ArgumentList(List args) { 
	this.args = args;
	if(args == null)
	    this.args = new LinkedList();
	setParents(args);
    }

    public void addLast(Expression e) {
        ((LinkedList)args).addLast(e);
	setParent(e);
    }

    public void add(int index, Expression e){
	args.add(index, e);
	setParent(e);
    }

    public void toSource() {
        for (Iterator i = args.iterator(); i.hasNext(); ) {
            Node n = (Node)i.next();
            n.toSource();
	    print(" ");
        }
    }

    public IR toIR() {
        return toIRList(args);
    }

    public List getList(){
	return args;
    }

    protected List args;

    /*** List interface methods ***/
    public void add(int index, Object element){
	args.add(index, element);
    }

    public boolean add(Object o){
	return args.add(o);
    }

    public boolean addAll(Collection c){
	return args.addAll(c);
    }

    public boolean addAll(int index, Collection c){
	return args.addAll(index, c);
    }

    public void clear(){
	args.clear();
    }

    public boolean contains(Object o){
	return args.contains(o);
    }

    public boolean containsAll(Collection c){
	return args.containsAll(c);
    }

    public boolean equals(Object o){
	return args.equals(o);
    }

    public Object get(int index){
	return args.get(index);
    }

    public int hashCode(){
	return args.hashCode();
    }

    public int indexOf(Object o){
	return args.indexOf(o);
    }

    public boolean isEmpty(){
	return args.isEmpty();
    }

    public Iterator iterator(){
	return args.iterator();
    }

    public int lastIndexOf(Object o){
	return args.lastIndexOf(o);
    }

    public ListIterator listIterator(){
	return args.listIterator();
    }

    public ListIterator listIterator(int index){
	return args.listIterator(index);
    }

    public Object remove(int index){
	return args.remove(index);
    }

    public boolean remove(Object o){
	return args.remove(o);
    }

    public boolean removeAll(Collection c){
	return args.removeAll(c);
    }

    public boolean retainAll(Collection c){
	return args.retainAll(c);
    }

    public Object set(int index, Object element){
	return args.set(index, element);
    }

    public int size(){
	return args.size();
    }

    public List subList(int fromIndex, int toIndex){
	return args.subList(fromIndex, toIndex);
    }

    public Object[] toArray(){
	return args.toArray();
    }

    public Object[] toArray(Object[] a){
	return args.toArray(a);
    }
}
