/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents a single array access.
 *
 * Specifically, expressions whose constructors are string 
 * constants.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.ArrayMonoList;

public class ArrayAccess extends Expression {
    /**
     * @param t  target of the array access
     * @param i  expression representing the index of the array access
     */
    public ArrayAccess( Expression t, Expression i ) { 
        constructor = "Array";
        setTarget(t);
        setIndex(i);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == target) {
            setTarget((Expression)new_child);
            return true;
        }
        else if (old_child == index) {
            setIndex((Expression)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setTarget( Expression t ) {
        target = t;
        setParent(target);
    }

    private void setIndex( Expression i ) {
        index = i;
        setParent(index);
    }

    public IR toIR() {
        return IR.make("getArray", new ArrayMonoList()
                .add(target.toIR())
                .add(index.toIR()));
    }

    public void toSource() {
        target.toSource();
        print("[");
        index.toSource();
        print("]");
    }

    public Type getType(ClassEnv loader) {
        Type t = target.getType(loader);
        return t.getElementType();
    }

    public String toString() {
        return target+"["+index+"]";
    }

    /**
     * Target of the array access.
     */
    public Expression target;
    /**
     * Expression representing the index of the array access.
     */
    public Expression index;
}
