/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents an array creation expression (without initializers).
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.ArrayMonoList;

public class ArrayCreation extends Expression {
    /**
     * @param t    type of the elements of the array
     * @param d    list of dimensions of the array
     */
    public ArrayCreation( Type t, List d ) { 
        setElementType(t);
        setDimensions(d);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == element_type) {
            setElementType((Type) new_child);
            return true;
        }
        else {
            return replaceInList(dimensions, old_child, new_child);
        }
    }

    protected boolean replaceUC(List old_child, List new_child) {
        if (old_child == dimensions) {
            setDimensions(new_child);
            return true;
        }
        return false;
    }

    private void setElementType( Type t ) {
        element_type = t;
        setParent(element_type);
    }

    private void setDimensions( List d ) {
        dimensions = d;
        setParents(dimensions);
    }

    public IR toIR() {
        return IR.make("newArray", new ArrayMonoList()
                .add(element_type.toIR())
                .add(toIRList(dimensions)));
    }

    public void toSource() {
        print("new ");
        element_type.toSource();
        print("[");
        printSourceList( dimensions, "][" );
        print("]");
    }

    public Type getType(ClassEnv loader) {
        if (element_type.isPrimitive()) 
            return new Type(element_type.getKind(), dimensions.size());
        else 
            return new Type(loader.getFQCN(element_type), dimensions.size());
    }

    /**
     * Type of the elements of the array
     */
    protected Type element_type;
    /**
     * List of dimensions of the array
     */
    protected List dimensions;
}
