/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents an assignment. 
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;
import uiuc.Jumbo.Util.ArrayMonoList;

import java.lang.reflect.Field;

public class AssignmentExpression extends BinaryExpression {
    /**
     * @param l  expression on the left hand side
     * @param t  type of assignment
     * @param r  expression on the right hand side
     */
    public AssignmentExpression( Expression l, String t, Expression r ) { 
        super( "", l, r );

        setOperator(t);
    }

    /**
     * @param l  name of variable on the left hand side
     * @param t  type of assignment
     * @param r  expression on the right hand side
     */
    public AssignmentExpression( String s, String t, Expression r ) { 
        super( "", new VarExpression(new Identifier(s)), r );
        setOperator(t);
    }

    public void setOperator(String t) {
        if (t == null || t.equals("")) operator = null;
        else operator = t;
    }

    public IR toIR() {
        MonoList args = new ArrayMonoList().add(left.toIR());
        if (operator != null) {
            try {
                Class instr = Class.forName("uiuc.Jumbo.Jaemus.Instr");
                Field f = instr.getField(operator);
                args.add(IR.make(f.getInt(null)));
            } catch (Exception e) {
                throw new Error("Error finding operator: "+e);
            }
        }
        args.add(right.toIR());

        return IR.make("putValue", args);
    }

    public void toSource() {
        if (!(parent instanceof Statement))
            oparen(); 
        left.toSource();
        print(" ");
        if (operator != null) print((String)BinaryExpression.ops.get(operator));
        print("= ");
        right.toSource();
        if (!(parent instanceof Statement))
            cparen();
    }

    public String toString() {
        return left.toString()+" "+(operator!=null?operator:"")+"= "+
            right.toString();
    }

    public Type getType(ClassEnv loader) {
	return right.getType(loader);
    }

    /** The operator before the equals sign, if any
    */
    protected String operator;
}
