/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents a binary operation expression.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;

import java.util.Hashtable;
import uiuc.Jumbo.Jaemus.Instr;
import uiuc.Jumbo.Util.*;
import uiuc.Jumbo.Util.ArrayMonoList;

import uiuc.Jumbo.Compiler.ConstVal;

public class BinaryExpression extends Expression  {
    /**
     * @param constr     constructor to use
     * @param l          Expression on the left side of the operator
     * @param r          Expression on the right side of the operator
     */
    public BinaryExpression( String constr, Expression l, Expression r ) { 
        super(constr);
        setLeft(l);
        setRight(r);
    }

    protected void setLeft(Expression l) {
        left = l;
        setParent(left);
    }

    protected void setRight(Expression r) {
        right = r;
        setParent(right);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == left) {
            setLeft((Expression)new_child);
            return true;
        }
        if (old_child == right) {
            setRight((Expression)new_child);
            return true;
        }
        return false;
    }

    Hashtable opnums = new Hashtable();
    {
        opnums.put("", new Integer(-1));
        opnums.put("MUL", new Integer(Instr.MUL));
        opnums.put("DIV", new Integer(Instr.DIV));
        opnums.put("REM", new Integer(Instr.REM));
        opnums.put("ADD", new Integer(Instr.ADD));
        opnums.put("SUB", new Integer(Instr.SUB));
        opnums.put("SHL", new Integer(Instr.SHL));
        opnums.put("SHR", new Integer(Instr.SHR));
        opnums.put("USHR", new Integer(Instr.USHR));
        opnums.put("CMP_GT", new Integer(Instr.CMP_GT));
        opnums.put("CMP_LT", new Integer(Instr.CMP_LT));
        opnums.put("CMP_GE", new Integer(Instr.CMP_GE));
        opnums.put("CMP_LE", new Integer(Instr.CMP_LE));
        opnums.put("CMP_EQ", new Integer(Instr.CMP_EQ));
        opnums.put("CMP_NE", new Integer(Instr.CMP_NE));
        opnums.put("AND", new Integer(Instr.AND));
        opnums.put("OR", new Integer(Instr.OR));
        opnums.put("XOR", new Integer(Instr.XOR));
    }

    public IR toIR() {
        if (constructor.equals("shortcutAnd") ||
            constructor.equals("shortcutAnd")) {
            return IR.make(constructor,
                           new ArrayMonoList()
                           .add(left.toIR())
                           .add(right.toIR()));
        }
        return IR.make("binOp",
                       new ArrayMonoList()
                       .add(IR.make(((Integer)opnums.get(constructor)).intValue()))
                       .add(left.toIR())
                       .add(right.toIR()));
    }

    static Hashtable ops = new Hashtable();
    static {
        ops.put("MUL", "*");
        ops.put("DIV", "/");
        ops.put("REM", "%");
        ops.put("ADD", "+");
        ops.put("SUB", "-");
        ops.put("SHL", "<<");
        ops.put("SHR", ">>");
        ops.put("USHR", ">>>");
        ops.put("CMP_GT", ">");
        ops.put("CMP_LT", "<");
        ops.put("CMP_GE", ">=");
        ops.put("CMP_LE", "<=");
        ops.put("CMP_EQ", "==");
        ops.put("CMP_NE", "!=");
        ops.put("AND", "&");
        ops.put("OR", "|");
        ops.put("XOR", "^");
        ops.put("shortcutAnd", "&&");
        ops.put("shortcutOr", "||");
    }

    public void toSource() {
	if (specialOutputFormatter != null &&
	    specialOutputFormatter.output(this))
	    return;
        if (!(parent instanceof Statement))
            oparen();
        left.toSource();
        print(ops.get(constructor));
        right.toSource();
        if (!(parent instanceof Statement))
            cparen();
    }

    /** Returns the type of the binary expression after promotions.
     * Is rather lax about type errors.
     */
    public Type getType(ClassEnv loader) {
        if (constructor.startsWith("shortcut")) 
            return new Type("boolean");
        int opnum = ((Integer)opnums.get(constructor)).intValue();
        // For the numeric values, see Jaemus.Instr
        if (opnum <= Instr.CMP_LE) return new Type("boolean");
        Type tl = left.getType(loader);
        Type tr = right.getType(loader);
        if (tl.isReference() || tr.isReference()) {
            return new Type("java.lang.String");
        } else {
            switch (opnum) {
            case -1:
                return tr;
            case Instr.ADD:
            case Instr.SUB:
            case Instr.MUL:
            case Instr.DIV:
            case Instr.REM:
            case Instr.NEG:
                if (tl.getKind() == Type.DOUBLE || tr.getKind() == Type.DOUBLE)
                    return new Type("double");
                if (tl.getKind() == Type.FLOAT || tr.getKind() == Type.FLOAT)
                    return new Type("float");
                // Notice fallthrough
            case Instr.SHL:
            case Instr.SHR:
            case Instr.USHR:
                if (tl.getKind() == Type.LONG || tr.getKind() == Type.LONG)
                    return new Type("float");
                return new Type("int");
            case Instr.AND:
            case Instr.OR:
            case Instr.XOR:
                if (tl.getKind() == Type.LONG || tr.getKind() == Type.LONG)
                    return new Type("float");
                if (tl.getKind() == Type.BOOLEAN || tr.getKind() == Type.BOOLEAN)
                    return new Type("boolean");
                return new Type("int");
            }
            throw new Error("getType not implemented for binaryExpression "+this);
        }
    }

    /* Using ConstVal here is a quick but ugly hack.  Thus, I keep
     * it here instead of spreading it to the other objects.
     */
    private ConstVal toConstVal(Literal l) {
        if (l instanceof IntegerLiteral)
            return ConstVal.make(((IntegerLiteral)l).getIntValue());
        if (l instanceof FloatLiteral)
            return ConstVal.make(((FloatLiteral)l).getFloatValue());
        if (l instanceof LongLiteral)
            return ConstVal.make(((LongLiteral)l).getLongValue());
        if (l instanceof DoubleLiteral)
            return ConstVal.make(((DoubleLiteral)l).getDoubleValue());
        if (l instanceof BooleanLiteral)
            return ConstVal.make(((BooleanLiteral)l).getBooleanValue());
        if (l instanceof StringLiteral)
            return ConstVal.make(((StringLiteral)l).getStringValue());
        if (l instanceof CharacterLiteral)
            return ConstVal.make(((CharacterLiteral)l).getCharValue());
        if (l instanceof NullLiteral)
            return ConstVal.makeNull();
        throw new Error("Unknown literal "+l+" of "+l.getClass());
    }

    private Literal fromConstVal(ConstVal c) {
        if (c instanceof ConstVal.IntConstVal)
            return new IntegerLiteral(c.intVal());
        if (c instanceof ConstVal.LongConstVal)
            return new LongLiteral(c.longVal());
        if (c instanceof ConstVal.FloatConstVal)
            return new FloatLiteral(c.floatVal());
        if (c instanceof ConstVal.DoubleConstVal)
            return new DoubleLiteral(c.doubleVal());
        if (c instanceof ConstVal.BooleanConstVal)
            return new BooleanLiteral(c.booleanVal());
        if (c instanceof ConstVal.StringConstVal)
            return new StringLiteral(c.stringVal());
        if (c instanceof ConstVal.NullConstVal)
            return new NullLiteral();
        throw new Error("Unknown constval "+c+" of "+c.getClass());
    }

    public Expression eval() {
	Integer opint = ((Integer)opnums.get(constructor));
	if (opint == null) {
	    System.out.println("Can't eval op "+constructor);
	    return this;
	}
        int op = opint.intValue();
        ConstVal c1 = toConstVal((Literal)left);
        ConstVal c2 = toConstVal((Literal)right);
        switch (op) {
        case Instr.MUL:
        case Instr.DIV:
        case Instr.REM:
        case Instr.ADD:
        case Instr.SUB:
            return fromConstVal(c1.mathBinOp(op, c2));
        case Instr.SHL:
        case Instr.SHR:
        case Instr.USHR:
        case Instr.AND:
        case Instr.OR:
        case Instr.XOR:
            return fromConstVal(c1.bitwiseBinOp(op, c2));
        case Instr.CMP_GT:
        case Instr.CMP_LT:
        case Instr.CMP_GE:
        case Instr.CMP_LE:
        case Instr.CMP_EQ:
        case Instr.CMP_NE:
            return fromConstVal(c1.compareBinOp(op, c2));
        case -1:
            throw new Error("Can't assign "+right+" to literal "+left);
        default: 
            throw new Error("Unknown operand in "+this);            
        }
    }

    public String toString() {
        return left+" "+ops.get(constructor)+" "+right;
    }

    /**
     * Left hand side of the operator.
     */
    protected Expression left;
    /**
     * Right hand side of the operator.
     */
    protected Expression right;
}
