/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.*;

/**
 * Represents the body of a class. 
 *
 * Segregates fields, methods, and constructors into separate lists.  
 *
 * Members must know how to add themselves to a ClassBody.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ClassBody extends Node implements ScopeMaker {
   /**
    * Creates a new, empty ClassBody.
    */
   public ClassBody() { 
       members = new ArrayList();
       setParents(members);
   }

    public void addMember(ClassBodyStatement cbs) {
        members.add(cbs);
        setParent((Node)cbs);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        return replaceInList(members, old_child, new_child);
    }

    /**
     * Prints out all declarations as source
     */
    public void toSource() {
        for (Iterator i = members.iterator(); i.hasNext(); ) {
            Node n = (Node)i.next();
            n.toSource();
            if (n instanceof FieldDeclaration) {
                print(";");
                linebreak();
            }
        }
    }

    public IR toIR() {
        return toIRList(members).flatten();
    }

    /**
     * List of members (methods, fields, inner classes, etc.
     */
    protected List members;

    /**
     * Parental class declaration this body is part of.
     * Set by ClassDeclaration/ClassDeclarationInBody itself.
     */
    protected Node parent;
}
