/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;

/**
 * Represents a class declaration (named, anonymous, inner, etc) in a class.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ClassDeclarationInBody extends ClassDeclaration 
    implements TypeDeclarationInBody
{
    /**
     * @param mods    Optional list of modifier flags
     * @param cname   Class name
     * @param sname   Optional superclass name
     * @param ifaces  Optional list of interface names
     * @param cbody   Class body (another class)
     */
    public ClassDeclarationInBody( ModifierList mods,
            Identifier cname,
            Identifier sname,
            List ifaces,
            ClassBody cbody) 
    { 
        super(mods, cname, sname, ifaces, cbody);
    }

    /* cache */
    private String fqcn;
    /** Returns the fully qualified class name for this class.
     * As an inner class, we must append it to the FQCN of the outer class.
     */
    public String getFQCN() {
	if (fqcn == null) {
	    Node p = parent;
	    while (p != null) {
		if (p instanceof ClassDefiner) {
		    fqcn = ((ClassDefiner)p).getFQCN()+"."+getName();
		    break;
		}
		p = p.parent;
	    }

	    if (fqcn == null)
		throw new Error("No enclosing class of inner class "+this+": "+getParents());
	}
	return fqcn;
    }

    public MethodDefiner findMethod(String name, List args, ClassEnv loader) {
	MethodDefiner md = super.findMethod(name, args, loader);
	if (md == null) {
	    System.out.println("Checking enclosing class "+getEnclosingClass());
            return getEnclosingClass().findMethod(name, args, loader);
        }
	return md;
    }

    public Type findMethodType(String name, List args,
                                            ClassEnv loader) {
        try {
            return super.findMethodType(name, args, loader);
        } catch (Error e) {
            return getEnclosingClass().findMethodType(name, args, loader);
        }
    }

    /** Outputs source code for this class definition.
     */
    public void toSource() {
        super.toSource();
        linebreak();
    }

}
