/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.*;

interface ClassDefiner {
    /** Find an AST node representing this field, if possible.
     * Searches super classes, but not outer classes.
     */
    FieldDefiner findField(String name, ClassEnv loader);
    /** Find an AST node representing this method, if possible.
     * Unlike findField, this does search outer classes.
     * For constructors (name == <init>), no super or outer search is done.
     */
    MethodDefiner findMethod(String name, List args,
                                 ClassEnv loader);
    /** Return the type of the named field in this class or superclasses.
     * Searches super classes, but not outer classes.
     * This throws an error if invoked on a compiled class (ClassWrapper).
     */
    Type findFieldType(String name, ClassEnv loader);
    /** Return the return type of the indicated method in this class 
     * or superclasses. Unlike findFieldType, this does search outer classes.
     * For constructors (name == <init>), no super or outer search is done.
     */
    Type findMethodType(String name, List args, ClassEnv loader);
    /** Return the FQCN of the super class.
     */
    String getSuperName();
    /** Return the FQCN of this class.
     */
    String getFQCN();
    /** Return a list of FQCNs of implemented/subclasses interfaces
     */
    List getInterfaces();
    /** Return true if this is a final class.
     */
    boolean isFinal();
    /** Return true if this is an interface rather than a full class */
    boolean isInterface();
}

