/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.*;

/**
 * Represents a class instance creation.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ClassInstanceCreation extends Expression {
    /**
     * @param c  name of the class being instanced
     * @param a  optional argument list
     */
    public ClassInstanceCreation( Identifier c, ArgumentList a ) { 
        setClassName(c);
        setArguments(a);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (class_name == old_child) {
            setClassName((Identifier)new_child);
            return true;
        }
        else {
            return replace(old_child, new_child);
        }
    }

    protected boolean replaceUC(List old_child, List new_child) {
        if (old_child == arguments) {
	    if(new_child instanceof ArgumentList)
		setArguments((ArgumentList)new_child);
	    else
		setArguments(new ArgumentList(new_child));
            return true;
        }
        return false;
    }

    private void setClassName(Identifier c) {
        class_name = c;
        setParent(class_name);
    }

    public void setArguments(ArgumentList l) {
        arguments = l;
        setParent(arguments);
    }

    public void addArgument(int index, Expression e) {
	arguments.add(index, e);
	setParent(e);
    }

    public IR toIR() {
	MonoList list = new ArrayMonoList()
	    //.add(IR.make(class_name.toString()));
	    .add(class_name.toIR());
	if(arguments == null)
	    list.add(IR.make(new ArrayMonoList()));
	else
	    list.add(arguments.toIR());
        return IR.make("newObject", list);
    }

    public void toSource() {
        print("new ");
        class_name.toSource();
        print("(");
        printSourceList(((arguments == null) ? null : arguments.getList()), ", ");
        print(")");
    }

    public String toString() {
        String r = "new "+class_name+"(";
        if (arguments != null) 
            for (Iterator i = arguments.getList().iterator(); i.hasNext(); ) {
                r += i.next().toString();
                if (i.hasNext()) r += ", ";
            }
        return r+")";
    }

    public Type getType(ClassEnv loader) {
        return loader.getFQCN(new Type(class_name));
    }

    /**
     * Name of the class of the instance being created
     */
    protected Identifier class_name;

    /**
     * Arguments.
     */
    protected ArgumentList arguments;
}
