/* Copyright (C) 2003, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents a full compilation unit, i.e. a file.
 * Really just a marker for the compilationUnit() combinator
 *
 * @author      Lars Clausen
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;

import uiuc.Jumbo.Util.*;
import uiuc.Jumbo.Compiler.*;

public class CompilationUnit extends Statement {
    public CompilationUnit(List l) {
	setContents(l);
    }

    public CompilationUnit(Node n) {
	if (n instanceof StatementList) {
	    setContents(((StatementList)n).statements);
	} else {
	    setContents(new ArrayList());
	    contents.add(n);
	}
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        return replaceInList(contents, old_child, new_child);
    }

    protected boolean replaceUC(List old_child, List new_child) {
        if (old_child == contents) {
	    setContents(new_child);
	    return true;
	}
	return false;
    }

    public void setContents(List n) {
	setParents(n);
	contents = n;
    }
    
    public IR toIR() {
        IRCode ir = IR.make("defineCompilationUnit", new ArrayMonoList());
	ir.addArg(toIRList(contents));
        return ir;
    }

    private List contents;
}
