/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;

import uiuc.Jumbo.Util.*;

/**
 * Represents the body of a constructor.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ConstructorBody extends StatementList
    implements ScopeMaker {
   /**
    * Creates a ConstructorBody with a constructor call
    * and a list of other body statements.
    *
    * @param c   explicit constructor call
    * @param l   rest of the body
    */
   public ConstructorBody( Statement c, StatementList l ) {
      statements = l.statements;
      setParents(statements);
      setConstructorCall(c);
   }

   /**
    * Creates a ConstructorBody with a list of statements.
    *
    * Adds the implicit super call.
    *
    * @param l   body of the constructor
    */
   public ConstructorBody( StatementList l ) { 
      this( new ExpressionStatement(new SuperConstructorInvocation(null)), l);
   }

   /**
    * Creates an empty ConstructorBody.
    */
   public ConstructorBody() {
      this( new StatementList() );
   }

    public void setConstructorCall(Statement c) {
        constructorcall = c;
        setParent(c);
    }

    public boolean replaceUC(Node old_child, Node new_child) {
        if (constructorcall == old_child) {
            setConstructorCall((Statement)new_child);
            return true;
        }
        return super.replaceUC(old_child, new_child);
    }

    /**
     * Prints out the body of a constructor as source.
     */
    public void toSource() {
        constructorcall.toSource();
        if (constructorcall.needsSemicolon()) print(";");
        if (statements != null && statements.size() != 0) {
            linebreak();
            for (Iterator i = statements.iterator(); i.hasNext(); ) {
                Statement s = (Statement)i.next();
                s.toSource();
                if (s.needsSemicolon())
                    print(";");
                if (i.hasNext())
                    linebreak();
            }
        }
    }

    public void addIR(IRCode ir) {
        ir.addArg(constructorcall.toIR());
        ir.addArg(super.toIR());
    }

   /**
    * ConstructorDeclaration this resides in.
    */
   protected ConstructorDeclaration const_decl;
   /**
    * Initial constructor call.
    */
   protected Statement constructorcall;
}
