/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;

import uiuc.Jumbo.Util.*;

/**
 * Represents a constructor declaration.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ConstructorDeclaration extends MethodDeclaration {
    /**
     * Creates a ConstructorDeclaration with a name and parameter 
     * list only.
     *
     * @param n name of the method.
     * @param p optional parameters of the method.
     */
    public ConstructorDeclaration( Identifier n, ParameterList p ) { 
	super( null, p );
    }
    public ConstructorDeclaration( Identifier n, List p ) { 
	super( null, new ParameterList(p) );
    }

    /**
     * @param b body of the constructor
     */
    public void setBody( StatementList b ) {
        super.setBody( b );

        ((ConstructorBody)body).const_decl = this;
    }

    public void toSource() {
        if ( modifiers != null ) { 
            modifiers.toSource();
        }

        ClassDeclaration cd = (ClassDeclaration)parent.parent;
        cd.name.toSource();

        oparen();
	if(parameters != null)
	    printSourceList( parameters.getList(), ", " );
        cparen();

        if (thrown_classes != null && thrown_classes.size() != 0) {
            print(" throws ");
            printSourceList( thrown_classes, ", " );
        }

        print(" ");
        ocurly();
        if ( body != null ) {
            linebreak();
            body.toSource();
            linebreak();
        }
        ccurly();
        linebreak();
    }


    public IR toIR() {
        IRCode ir = IR.make("defineConstructor", new ArrayMonoList());
        ir.addArg(modifiers != null?modifiers.toIR():IR.make(0));
	if(parameters == null)
	    ir.addArg(IR.make(new ArrayMonoList()));
	else
	    ir.addArg(parameters.toIR());
        ir.addArg(toIRList(thrown_classes));
        if (body != null) {
            ((ConstructorBody)body).addIR(ir);
        }
        else
            ir.addArg(IR.make("statements", new ArrayMonoList()));
        return ir;
    }

    public String getName() {
	return "<init>";
    }

    /** Return true if the method is declared final.
     * Constructors are by definition not final.
     */
    public boolean isFinal() { return false; }

    /** Return true if the method is declared static.
     * Constructors are by definition not static (they have a 'this').
     */
    public boolean isStatic() { return false; }

    /** Return the return type of the method.
     * Constructors are not considered to have a type.
     */
    public Type getType() { return null; }
}
