/* Copyright (C) 2003, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * A fully qualified class name (not containing variables or fields).
 * This can be used to replace a QNE or part of one to make it easier to
 * handle the AST.
 *
 * @author      Lars Clausen
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class FQCNExpression extends QualifiedNameExpression  {
    /**
     * @param n  the name of the expression
     */
    public FQCNExpression( Identifier n ) { 
	super(n);
    }

    public FQCNExpression( String s) {
	super(new Identifier(Util.unjoin(s, ".")));
    }

    public IR toIR() {
        return IR.make("get" + constructor, new ArrayMonoList()
                        .add(name.toIRString()));
    }

    public boolean replaceUC(Node old_child, Node new_child) {
	if (name == old_child) {
	    setName((Identifier)new_child);
	    return true;
	}
	return false;
    }

    private void setName(Identifier i) {
        name = i;
        setParent(i);
    }

    public void toSource() {
        name.toSource();
    }

    public String toString() {
        return name.toString();
    }

    /**
     * Returns a new type object based on this name.
     */
    public Type toType() {
        return new Type( name );
    }

    /** Get the variable declaration for this FQCNExpression
     * without depending on outside variable lookup.  FQCN has no such.
     */
    public Variable getVarDecl(Node from, ClassEnv loader) {
	return null;
    }

    /** Returns the type of the beginning variable or field.
     * FQCN has no such.
     */
    public Type getFieldOrVarType(Node from,
                                  Identifier name, ClassEnv loader) {
	return null;
    }

    /** Returns the first variable or field with the given name, or null.
     * This function scans through outer classes too.  
     * FQCN has no such.
     */
    public QNEMarker getFirstField(ClassEnv loader) {
	QNEMarker marker = new QNEMarker();
	marker.field = name.size();
	marker.type = getType(loader);
	return marker;
    }

    public Type getType(ClassEnv loader) {
	if (type == null) {
	    String classname = Util.conjoin(name.identifier_name, ".");
	    ClassDefiner cd = loader.getClass(classname);
	    type = new Type(cd.getFQCN());
	}
	return type;
    }

    /** A cached Type for this FQCN */
    private Type type;
}
