/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import java.util.Iterator;

import uiuc.Jumbo.Util.*;

/**
 * Represents a field declaration or a line of field declarations. 
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class FieldDeclaration extends Node implements ClassBodyStatement {
    /**
     * @param mods     Optional list of modifier flags
     * @param t        Type of the field being declared
     * @param vds      List of variable declarators
     */
    public FieldDeclaration( ModifierList mods, Type t, List vds ) { 
        constructor = "defineField";

        modifiers = mods;
        setParent(modifiers);
        variable_declarations = vds;
        setParents(vds);

        setType(t);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == type) {
            setType((Type)new_child);
            return true;
        }
        else {
            if (replaceInList(variable_declarations, old_child, new_child)) {
                if (new_child != null)
		    ((VariableDeclaration)new_child).variable.setType(type.copyType());
                return true;
            }
            else {
                return false;
            }
        }
    }

    private void setType(Type t) {
        type = t;
        setParent(type);

        Iterator iter = variable_declarations.iterator();
        while( iter.hasNext() ) {
            ((VariableDeclaration)iter.next()).variable.setType(type.copyType());
        }
    }

    /**
     * Writes source for a list of fields, by printing out the modifier flags 
     * and each VariableDeclaration.
     *
     */
    public void toSource() {
        Iterator iter = variable_declarations.iterator();

        while( iter.hasNext() ) {
            printDeclarationSource( (VariableDeclaration)iter.next() );
            if (iter.hasNext()) {
                print(";");
                linebreak();
            }
        }
    }


    public IR toIR() {
        MonoList list = new ArrayMonoList();
        for (Iterator i = variable_declarations.iterator(); i.hasNext(); ) {
            VariableDeclaration vd = (VariableDeclaration)i.next();
            IRCode ir = IR.make(constructor, new ArrayMonoList()
                                .add(modifiers!=null?modifiers.toIR():IR.make(0)));
            list.add(vd.addIR(ir));
        }
        return IR.make(list);
    }

    /**
     * Prints out a single field declaraction as source
     */
    public void printDeclarationSource( VariableDeclaration vd ) {
        if (modifiers != null)
            modifiers.toSource();
        vd.variable.type.toSource();
        print(" ");
        vd.toSource();
    }

    /** Returns the node defining the given field, if any.
    */
    public Variable findField(String field) {
        for (Iterator i = variable_declarations.iterator(); i.hasNext(); ) {
            VariableDeclaration vd = (VariableDeclaration)i.next();
            if (vd.variable.name.toString().equals(field))
                return vd.variable;
        }
        return null;
    }

    public boolean isFinal() {
        if (modifiers == null) return false;
        return modifiers.modifiers.contains("FINAL");
    }

    public boolean isStatic() {
        if (modifiers == null) return false;
        return modifiers.modifiers.contains("STATIC");
    }

    /**
     * List of modifier flags
     */
    protected ModifierList modifiers;
    /**
     * Type of the field declaration(s)
     */
    protected Type type;
    /**
     * Fields being declared.
     */
    protected List variable_declarations;
}
