/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * Represents a Java import declaration.
 * 
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ImportDeclaration extends Statement implements ImportDeclarator {
    /**
     * Creates a class import declaration.
     *
     * @param cname  name of the imported class
     */
    public ImportDeclaration( Identifier cname ) { 
        this(cname, false);
    }

    /**
     * Creates either a class or a package import declaration.
     *
     * @param cname  name of the imported class or package
     * @param isp    flag indicating if the importee is a package
     */
    public ImportDeclaration( Identifier cname, boolean isp ) { 
        class_name = cname;
        is_package = isp;
    }

    public IR toIR() {
        String constructor;
        if (is_package) {
            constructor = "importPackage";
        }
        else {
            constructor = "importClass";
        }

        return IR.make(constructor, new ArrayMonoList() 
                       .add(IR.make(class_name.toString())));
    }

    public Identifier getName() {
        return class_name;
    }

    public boolean isPackage() {
        return is_package;
    }

    public void toSource() {
        print("import ");
        class_name.toSource();
        if (is_package) {
            print( ".*" );
        }
    }

    /**
     * Name of the imported class.
     */
    protected Identifier class_name;
    /**
     * Flag indicating whether this is just a class 
     * or a package import.
     */
    protected boolean is_package;
}

interface ImportDeclarator {
    Identifier getName();
    boolean isPackage();
}
