/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

import java.util.*;
import java.util.Iterator;

/**
 * Represents an interface declaration in a class.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class InterfaceDeclarationInBody extends InterfaceDeclaration
    implements TypeDeclarationInBody {
    /**
     * @param mods    Optional list of modifier flags
     * @param iname   Interface name
     * @param ifaces  Optional list of interface names extended
     * @param ibody   Interface body (another class)
     */
    public InterfaceDeclarationInBody( ModifierList mods, 
                                       Identifier iname, 
                                       LinkedList ifaces, 
                                       InterfaceBody ibody) 
    { 
        super(mods, iname, ifaces, ibody);
    }

    public void toSource() {
        super.toSource();
        linebreak();
    }

    public MethodDefiner findMethod(String name, List args,
                                        ClassEnv loader) {
        try {
            return super.findMethod(name, args, loader);
        } catch (Error e) {
            return getEnclosingClass().findMethod(name, args, loader);
        }
    }

    public Type findMethodType(String name, List args,
                               ClassEnv loader) {
        try {
            return super.findMethodType(name, args, loader);
        } catch (Error e) {
            return getEnclosingClass().findMethodType(name, args, loader);
        }
    }

    /** Returns the fully qualified class name for this interface.
     * As an inner class, we must append it to the FQCN of the outer class.
     */
    public String getFQCN() {
        Node p = parent;
        while (p != null) {
            if (p instanceof ClassDefiner) {
                return ((ClassDefiner)p).getFQCN()+"."+getName();
            }
        }
        throw new Error("No enclosing class of inner interface "+this);
    }
}
