/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

/**
 * Class for any kind of literal (integers, floats, etc).
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public abstract class Literal extends Expression {
    /**
     * @param constr   constructor for this literal 
     * @param val      Value of the literal
     */
    public Literal( String constr, Object val ) { 
        super(constr);
        value = val;
    }

    public static Literal make( Object val ) {
        if (val == null) return new NullLiteral();
        if (val instanceof Byte) return new IntegerLiteral(((Byte)val).intValue());
        if (val instanceof Short) return new IntegerLiteral(((Short)val).intValue());
        if (val instanceof Integer) return new IntegerLiteral((Integer)val);
        if (val instanceof Float) return new FloatLiteral((Float)val);
        if (val instanceof Double) return new DoubleLiteral((Double)val);
        if (val instanceof Long) return new LongLiteral((Long)val);
        if (val instanceof Boolean) return new BooleanLiteral((Boolean)val);
        if (val instanceof Character) return new CharacterLiteral((Character)val);
        if (val instanceof String) return new StringLiteral((String)val);
        System.err.println("Can't handle literal of type "+val.getClass());
        return null;
    }

    /**
     * An optional letter indicating the type of the literal value.
     */
    public String typeMarker() {
        return "";
    }

    /**
     * Returns <0, 0 or >0 depending on whether this is <, = or > l
     */
    public int compare(Literal l) {
        throw new Error("Can't compare "+this+" "+this.getClass());
    }

    public abstract boolean equals(Object o);

    public Type getType(ClassEnv loader) {
        throw new Error("getType() not implemented for Literal of type "+getClass()+" yet");
    }

    /**
     * Value of this listeral
     */
    protected Object value;

}
