/* Copyright (C) 2003, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Represents a wrapped compiled method.
 *
 * @author      Lars Clausen
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

package uiuc.Jumbo.Ast;

import java.lang.reflect.*;

public class MethodWrapper implements MethodDefiner {
    public MethodWrapper(Method m) {
	method = m;
    }
    /** Returns the class defining this method */
    public ClassDefiner getDefiningClass() {
	return new ClassWrapper(method.getDeclaringClass());
    }
    /** Returns true if this method is declared final. */
    public boolean isFinal() {
	return (method.getModifiers() & Modifier.FINAL) != 0;
    }
    /** Returns trye if this method is declared static. */
    public boolean isStatic() {
	return (method.getModifiers() & Modifier.STATIC) != 0;
    }
    /** Returns the return type of this method. */
    public Type getType() {
	return Type.fromClass(method.getReturnType());
    }
    /** Returns the name of the method. */
    public String getName() {
	return method.getName();
    }
    
    private Method method;
}
