/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.*;

/**
 * Represents a holder for the list of parameters passed to a method 
 * when it's declared
 *
 * @author      Baris Aktemur
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ParameterList extends Node implements List{
    public ParameterList() { 
	parameters = new LinkedList();
	setParents(parameters);
    }

    public ParameterList(List parameters) { 
	this.parameters = parameters;
	if(parameters == null)
	    this.parameters = new LinkedList();
	setParents(parameters);
    }

    public void addLast(Parameter p) {
        ((LinkedList)parameters).addLast(p);
	setParent(p);
    }

    public void add(int index, Parameter p){
	parameters.add(index, p);
	setParent(p);
    }

    public void toSource() {
        for (Iterator i = parameters.iterator(); i.hasNext(); ) {
            Node n = (Node)i.next();
            n.toSource();
	    print(" ");
        }
    }

    public IR toIR() {
        return toIRList(parameters);
    }

    public List getList(){
	return parameters;
    }

    protected List parameters;

    /*** List interafe methods ***/
    public void add(int index, Object element){
	parameters.add(index, element);
    }

    public boolean add(Object o){
	return parameters.add(o);
    }

    public boolean addAll(Collection c){
	return parameters.addAll(c);
    }

    public boolean addAll(int index, Collection c){
	return parameters.addAll(index, c);
    }

    public void clear(){
	parameters.clear();
    }

    public boolean contains(Object o){
	return parameters.contains(o);
    }

    public boolean containsAll(Collection c){
	return parameters.containsAll(c);
    }

    public boolean equals(Object o){
	return parameters.equals(o);
    }

    public Object get(int index){
	return parameters.get(index);
    }

    public int hashCode(){
	return parameters.hashCode();
    }

    public int indexOf(Object o){
	return parameters.indexOf(o);
    }

    public boolean isEmpty(){
	return parameters.isEmpty();
    }

    public Iterator iterator(){
	return parameters.iterator();
    }

    public int lastIndexOf(Object o){
	return parameters.lastIndexOf(o);
    }

    public ListIterator listIterator(){
	return parameters.listIterator();
    }

    public ListIterator listIterator(int index){
	return parameters.listIterator(index);
    }

    public Object remove(int index){
	return parameters.remove(index);
    }

    public boolean remove(Object o){
	return parameters.remove(o);
    }

    public boolean removeAll(Collection c){
	return parameters.removeAll(c);
    }

    public boolean retainAll(Collection c){
	return parameters.retainAll(c);
    }

    public Object set(int index, Object element){
	return parameters.set(index, element);
    }

    public int size(){
	return parameters.size();
    }

    public List subList(int fromIndex, int toIndex){
	return parameters.subList(fromIndex, toIndex);
    }

    public Object[] toArray(){
	return parameters.toArray();
    }

    public Object[] toArray(Object[] a){
	return parameters.toArray(a);
    }
}
