/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.*;

/**
 * Quoted code fragments.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 * 
 */

public class QuotedCode extends Expression {
    /**
     * Creates a quoted code fragment.
     *
     * @param n quoted code (ast subtree).
     */
    public QuotedCode( Object n ) {
	setCode(n);
    }

    public void setCode(Object n) {
        code = n;
    }

    public void toSource() {
        if (code instanceof Node) {
            IR ir = ((Node)code).toIR();
            print(ir.toSource());
        } else if (code instanceof LinkedList) {
            IRList irlist = toIRList(((LinkedList)code));
            print(irlist.toSource());
        } else {
            throw new Error("Bad type of quoted code in toSource: "+code.getClass());
        }
    }

    public IR toIR() {
        try {
            if (code instanceof Node) {
                IR ir = ((Node)code).toIR();
                return ir.lift();
            } else if (code instanceof LinkedList) {
                IR irlist = toIRList(((LinkedList)code));
                return irlist.lift();
            } else {
                throw new Error("Bad type of quoted code in toIR: "+code.getClass());
            }
        } catch (Exception e) {
            System.out.println("Exception while quoting "+code);
            System.out.println(e);
            e.printStackTrace();
            throw new Error("Quote error");
        }
    }

    public Type getType(ClassEnv loader) {
        return new Type("uiuc.Jumbo.Compiler.Code");
    }

    protected Object code;
}
