/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * ScopedStatementList.java
 *
 *
 */

package uiuc.Jumbo.Ast;

import java.util.Iterator;
import uiuc.Jumbo.Util.*;
import uiuc.Jumbo.Compiler.*;

/**
 * A list of statements within a scope.
 * @version      1.0, 10 Jan 2002
 * @author       Lars Clausen
 *
*/

public class ScopedStatementList extends StatementList 
    implements ScopeMaker {
    public void toSource() {
        ocurly();
        linebreak();
        Iterator i = statements.iterator();
        while (i.hasNext()) {
            Statement s = (Statement)i.next();
            s.toSource();
            if (s.needsSemicolon())
                print(";");
            linebreak();
        }
        ccurly();
    }
    public IR toIR() {
        IRCode ir = IR.make("scopedStatements", new ArrayMonoList());
        if (use_singles && statements.size() == 1) {
            ir.addArg(((Node)statements.get(0)).toIR());
        } else {
            ir.addArg(toIRList(statements));
        }
        return ir;
    }

    public boolean needsSemicolon() { return false; }

    public String toString() {
        String s = "{";
        for (Iterator i = statements.iterator(); i.hasNext(); ) {
            s += i.next();
            if (i.hasNext());
            s += ", ";
        }
        s += "}";
        return s;
    }
}

/* End of class ScopedStatementList*/
