/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;
import uiuc.Jumbo.Util.*;

/**
 * Represents a string literal. 
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class StringLiteral extends Literal {
    /**
     * @param v   value of the literal
     */
    public StringLiteral( String v ) { 
        super( "", v );
    }
    
    public IR toIR() {
        return IR.make("stringConstant", new ArrayMonoList()
                        .add(IR.make((String)value)));
    }

    public String makeQuoted(String s) {
        StringBuffer b = new StringBuffer(s.length()*2);
        for (int i = 0; i < s.length(); i++) {
            switch (s.charAt(i)) {
            case '\b': b.append( "\\b" ); break;
            case '\t': b.append( "\\t" ); break;
            case '\n': b.append( "\\n" ); break;
            case '\f': b.append( "\\f" ); break;
            case '\r': b.append( "\\r" ); break;
            case '\'': b.append( "\\'" ); break;
            case '\"': b.append( "\\\"" ); break;
            case '\\': b.append( "\\\\" ); break;
            default: b.append(s.charAt(i));
            }
        }
        return b.toString();
    }

    public void toSource() {
        print("\""+makeQuoted((String)value)+"\"");
    }
    public int compare(Literal l) {
        return ((String)value).compareTo((String)l.value);
    } 
    public boolean equals(Object o) {
        if (o instanceof StringLiteral) {
            return value.equals(((StringLiteral)o).value);
        }
        return false;
    }
   public Type getType(ClassEnv loader) {
        return new Type("java.lang.String");
    }
    public String toString() {
        return "\""+value+"\"";
    }
    public String getStringValue() {
        return (String)value;
    }
}
