/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a synchronized block.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class SynchronizedStatement extends Statement {
   /**
    * @param e  expression to synchronize on
    * @param b  block being synchronized
    */
   public SynchronizedStatement( Expression e, StatementList b ) { 
       setExpression(e);
       setBlock(b);
   }

   private void setExpression(Expression e) {
       expression = e;
       setParent(expression);
   }

   private void setBlock(StatementList b) {
       block = b;
       setParent(block);
   }

   protected boolean replaceUC(Node old_child, Node new_child) {
       if (old_child == expression) {
           setExpression((Expression)new_child);
           return true;
       }
       else if (old_child == block) {
           setBlock((StatementList)new_child);
           return true;
       }
       else {
           return false;
       }
   }

    public IR toIR() { 
        return IR.make("synchronizedBlock", new ArrayMonoList() 
                       .add(expression.toIR())
                       .add(block.toIR()));
    }

    public void toSource() {
        print("synchronized (");
        expression.toSource();
        print(") ");
        block.toSource();
    }

    public boolean needsSemicolon() { return false; }

   /**
    * Expression to be synchronized on.
    */
   protected Expression expression;
   /**
    * Block to be synchronized.
    */
   protected StatementList block;
}
