/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a reference to the special 'this' identifier.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class ThisExpression extends Expression  {
    public ThisExpression() { 
	super("Constructor.getThis()");
    }

    public ThisExpression(Identifier name) { 
	super("Constructor.getThis");
	setName(name);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == name) {
	    setName((Identifier)new_child);
	    return true;
	}
	return false;
    }

    public void setName(Identifier name) {
	this.name = name;
	setParent(name);
    }

    public IR toIR() {
	if (name == null)
	    return IR.make("getThis", new ArrayMonoList());
	else 
	    return IR.make("getThis", new ArrayMonoList()
			   .add(IR.make(name)));
    }

    public void toSource() {
        if (name != null)
	    print(name+".this");
	else
	    print("this");
    }

    public String toString() {
        if (name != null)
	    return name+".this";
	else
	    return "this";
    }
    public Type getType(ClassEnv loader) {
	if (name != null) {
	    ClassDefiner cd = loader.getClass(name.getString());
	    if (cd == null) System.out.println("Can't find class "+name.getClass()+": "+name.identifier_name);
	    return new Type(cd.getFQCN());
	} else {
	    ClassDefiner cd = getThis();
	    if (cd instanceof AnonymousClassCreation)
		return new Type(cd.getSuperName());
	    return new Type(cd.getFQCN());
	}
    }

    public ClassDefiner getClass(ClassEnv loader) {
	if (name != null) {
	    return loader.getClass(name.getString());
	} else {
	    return getThis();
	}
    }

    /** Note that this is a simple equals, it does not attempt to resolve
     * class names.
     */
    public boolean equals(Object o) {
	if (!(o instanceof ThisExpression)) return false;
	if (name == null) return ((ThisExpression)o).name == null;
	return name.equals(((ThisExpression)o).name);
    }

    /** The name that identifies the 'this' class, or null for a plain 'this'
     */
    protected Identifier name;
}
