/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import java.util.*;
import uiuc.Jumbo.Util.*;

/**
 * Represents a try-catch-finally statement.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class TryCatchStatement extends Statement {
   /**
    * @param tb     try block
    * @param cl     optional list of catch blocks
    * @param f      finally block
    */
   public TryCatchStatement( StatementList tb, List cl,
                             StatementList f ) { 
       setTryBlock(tb);
       setCatches(cl);
       setFinallyBlock(f);
   }

    public boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == try_block) {
            setTryBlock((StatementList)new_child);
            return true;
        }
        if (old_child == finally_block) {
            setFinallyBlock((StatementList)new_child);
            return true;
        }
        return replaceInList(catches_list, old_child, new_child);
    }

    public boolean replaceUC(List old_child, List new_child) {
        if (old_child == catches_list) {
            setCatches(new_child);
            return true;
        }
        return false;
    }

    public void setTryBlock(StatementList tb) {
        try_block = tb;
        setParent(tb);
    }

    public void setFinallyBlock(StatementList fb) {
        finally_block = fb;
        setParent(fb);
    }

    public void setCatches(List l) {
        catches_list = l;
        setParents(l);
    }

    public IR toIR() {
        if (finally_block != null) {
            return IR.make("handlersWithFinally", new ArrayMonoList()
                           .add(try_block.toIR())
                           .add(toIRList(catches_list))
                           .add(finally_block.toIR()));
        } else {
            return IR.make("handlers", new ArrayMonoList()
                           .add(try_block.toIR())
                           .add(toIRList(catches_list)));
        }
    }

    public void toSource() {
        print("try ");
        try_block.toSource();
        printSourceList(catches_list);
        if (finally_block != null) {
            print("finally ");
            finally_block.toSource();
        }
    }
    public boolean needsSemicolon() { return false; }

   /**
    * Try block.
    */
   protected StatementList try_block;
   /**
    * List of catch blocks.
    */
   protected List catches_list;
   /**
    * Finally block.
    */
   protected StatementList finally_block;
}
