/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a unary operation expression.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class UnaryExpression extends Expression  {
    /**
     * @param constr     constructor to use
     * @param e          Expression being operated upon
     */
    public UnaryExpression( String constr, Expression e ) { 
        super(constr);
        setExpression(e);
    }

    protected boolean replaceUC(Node old_child, Node new_child) {
        if (old_child == expression) {
            setExpression((Expression)new_child);
            return true;
        }
        else {
            return false;
        }
    }

    private void setExpression(Expression e) {
        expression = e;
        setParent(expression);
    }

    public IR toIR() {
        return IR.make(constructor, new ArrayMonoList()
                       .add(expression.toIR()));
    }

    public void toSource() {
        if (constructor.equals("negate")) print("-");
        else if (constructor.equals("bitwiseComplement")) print("~");
        else if (constructor.equals("booleanNot")) print("!");
	else print("<UNARY: "+constructor+">");
        expression.toSource();
    }

    public Type getType(ClassEnv loader) {
        return expression.getType(loader);
    }

    public Expression eval() {
        if (!(expression instanceof Literal)) return expression;
        if (constructor.equals("negate")) {
            if (expression instanceof CharacterLiteral)
                return new IntegerLiteral(-((CharacterLiteral)expression).getCharValue());
            if (expression instanceof IntegerLiteral)
                return new IntegerLiteral(-((IntegerLiteral)expression).getIntValue());
            if (expression instanceof FloatLiteral)
                return new FloatLiteral(-((FloatLiteral)expression).getFloatValue());
            if (expression instanceof LongLiteral)
                return new LongLiteral(-((LongLiteral)expression).getLongValue());
            if (expression instanceof DoubleLiteral)
                return new DoubleLiteral(-((DoubleLiteral)expression).getDoubleValue());
            System.err.println("Can't negate "+expression);
            return this;
        }
	else if (constructor.equals("bitwiseComplement")) {
	    if (expression instanceof CharacterLiteral)
                return new IntegerLiteral(~((CharacterLiteral)expression).getCharValue());
            if (expression instanceof IntegerLiteral)
                return new IntegerLiteral(~((IntegerLiteral)expression).getIntValue());
            if (expression instanceof LongLiteral)
                return new LongLiteral(~((LongLiteral)expression).getLongValue());
            System.err.println("Can't complement "+expression);
            return this;

	} else if (constructor.equals("booleanNot")) {
	    if (expression instanceof BooleanLiteral) {
		return new BooleanLiteral(!((Boolean)((BooleanLiteral)expression).value).booleanValue());
	    }
            System.err.println("Can't not "+expression);
            return this;
	}
        return expression;
    }

    /**
     * Expression being acted upon.
     */
    protected Expression expression;
}
