/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a variable declaration, contained within either a field 
 * definition/declaration or a local variable declaration.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class VariableDeclaration extends Node {
   /**
    * @param var    Variable being declared
    * @param init   Optional initializer
    */
   public VariableDeclaration( Variable var, Expression init ) { 
       setVariable(var);
       setInitializer(init);
   }

   protected boolean replaceUC(Node old_child, Node new_child) {
       if (old_child == variable) {
           setVariable((Variable)new_child);
           return true;
       }
       else if (old_child == initializer) {
           setInitializer((Expression)new_child);
           return true;
       }
       else {
           return false;
       }
   }

   private void setVariable(Variable var) {
       variable = var;
       setParent(variable);
   }

   private void setInitializer(Expression init) {
       initializer = init;
       setParent(initializer);
   }

    /** Adds the name and initializer to a declaration IR */
    public IR addIR(IRCode base) {
        IRCode full = base
            .addArg(variable.type.toIR())
            .addArg(variable.name.toIRString());
        
        if (initializer != null)
            full.addArg(initializer.toIR(variable.type));
        return full;
    }

    /**
     * Prints out the variable's declaration constructor as source.
     */
    public void toSource() {
        variable.toDeclarationSource();
        if (initializer != null) {
            print(" = ");
            initializer.toSource();
        }
    }

    public String toString() {
        String res = variable.toString();
        if (initializer != null) {
            res += " = "+initializer;
        }
        return res;
    }

    public Variable getContainedVariableDeclaration(String name) {
        Variable v = variable.getContainedVariableDeclaration(name);
	if (trace_vardecl)
	    System.out.println("VarDecl "+v+" for "+name+" in "+this);
	return v;
    }

    /**
    * Variable being declared.
    */
   protected Variable variable;
   /**
    * Initializer of the variable.
    */
   protected Expression initializer;
}
