/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package uiuc.Jumbo.Ast;

import uiuc.Jumbo.Util.*;

/**
 * Represents a while statement.
 *
 * @author      Ava Arachne Jarvis
 * @version     %I%, %G%
 * @since       JDK 1.3
 */

public class WhileStatement extends Statement implements Labelable {
   /**
    * @param te   test expression
    * @param b    body statement of the while loop
    */
   public WhileStatement( Expression te, Statement b ) { 
       setTest(te);
       setBody(b);
       constructor = "whileDo";
       label_constructor = "whileLabel";
       is_loop = true;
   }

   protected boolean replaceUC(Node old_child, Node new_child) {
       if (old_child == test) {
           setTest((Expression)new_child);
           return true;
       }
       else if (old_child == body) {
           setBody((Statement)new_child);
           return true;
       }
       else {
           return false;
       }
   }

   private void setTest(Expression te) {
       test = te;
       setParent(test);
   }

   private void setBody(Statement b) {
       body = b;
       setParent(body);
   }

    public IR toIR() {
        return IR.make(constructor, new ArrayMonoList()
                       .add(test.toIR())
                       .add(body.toIR()));
    }

    public IR toIR( String label) {
        return IR.make(label_constructor, new ArrayMonoList()
                       .add(IR.make(label))
                       .add(test.toIR())
                       .add(body.toIR()));
    }

    public void toSource() {
	if( specialOutputFormatter != null &&
	    specialOutputFormatter.output(this))
	    return;
	else
	{
	    print("while (");
	    test.toSource();
	    print(") ");
	    if (body instanceof EmptyStatement)
		print(";");
	    else
		body.toSource();
	}
    }

    public String toString() {
	String res = "while (" + test + ") ";
	if (body instanceof EmptyStatement)
	    res += ";";
	else
	    res += body.toString();
	return res;
    }

    public boolean needsSemicolon() { return false; }

    public String getLabel() {
        if (parent instanceof LabeledStatement) {
            return ((LabeledStatement)parent).getLabel();
        }
        return "";
    }

   /**
    * Test expression.
    */
   protected Expression test;
   /**
    * Body of the loop.
    */
   protected Statement body;
   /**
    * Constructor if this happens to be a labeled statement.
    */
   protected String label_constructor;
}
