/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * EmptyLinkedMonoList.java
 *
 *
 */

package uiuc.Jumbo.Util;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * An empty linked-list MonoList.
 * * @version      1.0,  3 Nov 2004
 * @author       Philip Morton
 *
*/

public final class EmptyLinkedMonoList implements MonoList {
    public EmptyLinkedMonoList() {
    }

	public MonoList add(final Object o) {
        return new LinkedMonoList(o);
    }
    public Iterator iterator() {
        return new EmptyLinkedMonoIterator(this);
    }
	
    /* Not side-effect free */
    class EmptyLinkedMonoIterator implements Iterator {
        EmptyLinkedMonoIterator(final EmptyLinkedMonoList t) {
        }
        public boolean hasNext() {
            return false;
        }
        public Object next() {
            throw new NoSuchElementException();
        }
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
    public Object foldr(FoldOp op, Object zero) {
		return zero;
    }
    public Object foldl(FoldOp op, Object zero) {
		return zero;
    }
    public Object get(final int i) {
        throw new IndexOutOfBoundsException(""+i);
    }

    public MonoList subList(final int from) {
        throw new IndexOutOfBoundsException(""+from);
    }
	
    public MonoList subList(final int from, final int to) {
        throw new IndexOutOfBoundsException(""+from);
	}
	
    public int size() {
		return 0;
    }
    public boolean isEmpty() {
        return true;
    }

	public MonoList addAll(MonoList l)
	{
		if(l.size() == 0)
			return this;
		Iterator i = l.iterator();
		MonoList foo = new LinkedMonoList(i.next());

		for(; i.hasNext(); )
		{
			foo = foo.add(i.next());
		}

		return foo;
	}

    public String toString() {
        return "[]";
    }

    /* Sideffecting */
    public Object[] toArray() {
        final Object[] array = new Object[0];
        return array;
    }

    public MonoList copy() {
        return this;
    }
}

/* End of class EmptyLinkedMonoList*/

