/* Copyright (C) 2002, University of Illinois.  All rights reserved. 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * MonoList.java
 *
 *
 */

package uiuc.Jumbo.Util;

import java.util.Iterator;

/**
 * A monotonically increasing list with random access.
 * @version      1.0,  8 Nov 2001
 * @author       Lars Clausen
 *
*/

public interface MonoList extends java.io.Serializable{
    /** Add a single object to the end of this list */
    public MonoList add(Object o);
    /** Add all objects from the given list to the end of this list */
    public MonoList addAll(MonoList l);
    /** Get the i'th object from this list */
    public Object get(int i);
    /** Iterator over this list */
    public Iterator iterator();
    /** Foldr function for this list.
     * For a list [a, b, c], foldr(#, x) returns a#(b#(c#x)), where
     * # is some operation. */
    public Object foldr(FoldOp op, Object zero);
    /** Foldl function for this list.
     * For a list [a, b, c], foldl(#, x) returns ((x#a)#b)#c, where
     * # is some operation. */
    public Object foldl(FoldOp op, Object zero);
    /** Get the size of this list (number of elements) */
    public int size();
    /** Get a part of this list.
     * Note that unlike the List interface, these subLists aren't
     * necessarily back by the parent (though they might share the
     * underlying array). */
    public MonoList subList(int from, int to);
    /** Get the current list as an array */
    public Object[] toArray();
    /** Get a sharing-free copy of the list */
    public MonoList copy();
}

/* End of interface MonoList*/
